function Create(self)
    --The Y coordinate at which to hover.
    self.hoverY = self.Pos.Y - 12;

    --The maximum speed of the train.
    self.maxspeed = 25;

    --The current speed of the train.
    self.curspeed = self.maxspeed;

    --If the train has crashed or not.
    self.crashed = false;

    --The time since the train has crashed.
    self.crashtimer = Timer();

    --This detects when to add some swaying motion to make movement seem more natural.
    self.swaytimer = Timer();

    --How much to sway (radians).
    self.sway = 0.25;

    --How far the train can rotate before crashing (radians)
    self.maxrot = 0.8;
	
	-- Add Turrets
	local turrets = nil

	if PresetMan:GetModuleID("Wehrmacht.rte") ~= -1 then
		self.UseWehrmacht = true
		self.UseAirSupport = false
		self.UseUniTec = false;
	else
		self.UseWehrmacht = false
	end--]]--
	
	if PresetMan:GetModuleID("UniTec.rte") ~= -1 then
		self.UseUniTec = true;
		self.UseWehrmacht = false
		self.UseAirSupport = false
	else
		self.UseUniTec = false;
	end--]]--

	if PresetMan:GetModuleID("Air Support.rte") ~= -1 then
		self.UseAirSupport = true
		self.UseWehrmacht = false
		self.UseUniTec = false;
	else
		self.UseAirSupport = false
	end--]]--

	self.Offset = Vector(0,6)

	weaps = {"M60", "Flak Cannon", "Gatling Gun"}
	weapsm = {"Ronin.rte" , "Coalition.rte", "Coalition.rte"}
	
	if self.UseUniTec then
		turrets = {"Sentry Gun Mini-G", "Sentry Gun A13-G", "Sentry Gun LAS-R-G"}
		self.Offset = Vector(0,0)
	end
	
	if self.UseAirSupport then
		turrets = {"50 cal Slave Turret", "Flak Slave Turret"}
		self.Offset = Vector(0,5)
	end

	if self.UseWehrmacht then
		turrets = {"88mm FlaK 36"}
		self.Offset = Vector(0,0)
	end
	
	self.turret = {}
	self.regen = {}
	
	for i = 1, 2 do
		self.regen[i] = false;
	
		self.turret[i] = nil
		-- Try to create unitec turret
		if self.UseUniTec then
			self.turret[i] = CreateACrab(turrets[math.random(#turrets)],"UniTec.rte");
		end

		if self.UseWehrmacht then
			self.turret[i] = CreateACrab(turrets[math.random(#turrets)],"Wehrmacht.rte");
		end
		
		if self.UseAirSupport then
			if not TRAIN_CHASE_SPAWN_CONTROL then
				TRAIN_CHASE_SPAWN_CONTROL = true
				self.turret[i] = CreateACrab("Turret Control","Air Support.rte");
			else
				self.turret[i] = CreateACrab(turrets[math.random(#turrets)],"Air Support.rte");
			end
		end
		
		-- Vanilla turret fallback
		if self.turret[i] == nil then
			if math.random() < 0.80 then
				self.regen[i] = true;
				
				local r = math.random(#weaps)
			
				self.turret[i] = CreateAHuman("Combat Robot", "Imperatus.rte");
				self.turret[i]:AddInventoryItem(CreateHDFirearm(weaps[r], weapsm[r]));
			else
				self.turret[i] = CreateACrab("Gatling Turret AA", "TrainChase.rte");
			end
		end
		
		if i == 1 then
			self.turret[i].Pos = Vector(self.Pos.X + 40,self.Pos.Y - 44) + self.Offset;
		end

		if i == 2 then
			self.turret[i].Pos = Vector(self.Pos.X - 40,self.Pos.Y - 44) + self.Offset;
		end
		
		self.turret[i].Vel = self.Vel;
		self.turret[i].Team = self.Team;
		MovableMan:AddActor(self.turret[i]);
	end
end

function Update(self)
    --If the train hasn't crashed...
    if self.crashed == false then
	--The following lines level out the train (until it has crashed).
	--This makes it hover.
	self.Vel.Y = self.Vel.Y - ((self.Pos.Y - self.hoverY)) / 35;

	--This makes sure it's hovering around the right spot.
	self.Pos.Y = ((self.Pos.Y * 9) + self.hoverY) / 10;

	--This keeps the train at a constant speed.
	--if self.Vel.X ~= 50 then
	--    self.Vel.X = (self.Vel.X * 9 + self.curspeed) / 10;
	--end

	--This keeps the train rotated in the right direction.
	if self.RotAngle ~= 0 then
	    self.AngularVel = (self.AngularVel * 74 - math.deg(self.RotAngle)) / 75;
	end

	--Crash the train if it's rotated too far.
	if self.RotAngle <= -self.maxrot or self.RotAngle >= self.maxrot then
	    self.crashed = true;
	    self.crashtimer:Reset();
	end
	
	--If the train is dead, crash it.
	if self.Health <= 10 then
	    self.crashed = true;
	    self.crashtimer:Reset();
	end

	--Every 5 seconds, check if the train has any sway.  If not, apply some.
	if self.swaytimer:IsPastSimMS(5000) then
	    self.swaytimer:Reset();
	    self.AngularVel = self.sway;
	end
    elseif self.crashtimer:IsPastSimMS(500) == true then
	--Explode five seconds after crashing.
	self:GibThis();
    end
	
	for i = 1, 2 do
		if MovableMan:IsActor(self.turret[i]) and MovableMan:IsActor(self) then
			if i == 1 then
				local x = math.sin(self.RotAngle + 1.57 * 1.53) * 60;
				local y = math.cos(self.RotAngle + 1.57 * 1.53) * 60;
				
				self.turret[i].Pos = Vector(self.Pos.X + x, self.Pos.Y + y) + self.Offset;
			end

			if i == 2 then
				local x = math.sin(self.RotAngle - 1.57 * 1.53) * 60;
				local y = math.cos(self.RotAngle - 1.57 * 1.53) * 60;
				
				self.turret[i].Pos = Vector(self.Pos.X + x, self.Pos.Y + y) + self.Offset;
			end

			self.turret[i].Vel = self.Vel;
			self.turret[i].RotAngle = self.RotAngle;
		end
		
		if not MovableMan:IsActor(self.turret[i]) and MovableMan:IsActor(self) then
			self.turret[i] = nil;
			if self.regen[i] then
				if math.random() < 0.005 then
					local r = math.random(#weaps)
					self.turret[i] = CreateAHuman("Soldier Heavy", "Coalition.rte");
					self.turret[i]:AddInventoryItem(CreateHDFirearm(weaps[r], weapsm[r]));
					if i == 1 then
						self.turret[i].Pos = Vector(self.Pos.X + 40,self.Pos.Y - 44) + self.Offset;
					end

					if i == 2 then
						self.turret[i].Pos = Vector(self.Pos.X - 40,self.Pos.Y - 44) + self.Offset;
					end
					
					self.turret[i].Vel = self.Vel;
					self.turret[i].Team = self.Team;
					MovableMan:AddActor(self.turret[i]);
				end
			end
		end
	end
end


function Destroy(self)
	for i = 1,2 do
		if MovableMan:IsActor(self.turret[i]) then
			self.turret[i].Health = 0
		end
	end
end