function Create(self)
    --The Y coordinate at which to hover.
    self.hoverY = self.Pos.Y - 12;

    --The maximum speed of the train.
    self.maxspeed = 25;

    --The current speed of the train.
    self.curspeed = self.maxspeed;

    --If the train has crashed or not.
    self.crashed = false;

    --The time since the train has crashed.
    self.crashtimer = Timer();

    --This detects when to add some swaying motion to make movement seem more natural.
    self.swaytimer = Timer();

    --How much to sway (radians).
    self.sway = 0.25;

    --The area in which the train should stop.
    self.stoparea = SceneMan.Scene:GetArea("Train Stop Area");

    --Whether the train has come to a stop at a destination.
    self.stopped = false;

    --How long the train has been stopped for.
    self.stoptimer = Timer();

    --How far the train can rotate before crashing (radians)
    self.maxrot = 1.8;
	
	-- Add Turrets
	self.turrets = nil
	
	if PresetMan:GetModuleID("UniTec.rte") ~= -1 then
		self.UseUniTec = true;
	else
		self.UseUniTec = false;
	end

	if PresetMan:GetModuleID("Air Support.rte") ~= -1 then
		self.UseAirSupport = true
	else
		self.UseAirSupport = false
	end
	
	if self.UseAirSupport and self.UseUniTec then
		if math.random() < 0.65 then
			self.UseUniTec = true;
			self.UseAirSupport = false;
		else
			self.UseAirSupport = true;
			self.UseUniTec = false;
		end
	end

	self.Offset = Vector(0,6)
	
	if self.UseUniTec then
		self.turrets = {"Sentry Gun Mini-G", "Sentry Gun A13-G", "Sentry Gun LAS-R-G"}
		self.Offset = Vector(0,0)
	end
	
	if self.UseAirSupport then
		self.turrets = {"50 cal Slave Turret", "Flak Slave Turret", "Turret Control"}
		self.Offset = Vector(0,5)
	end
	
	self.turret = {}
	
	for i = 1, 2 do
		self.turret[i] = nil
		-- Try to create unitec turret
		if self.UseUniTec then
			self.turret[i] = CreateACrab(self.turrets[math.random(#self.turrets)],"UniTec.rte");
		end
		
		if self.UseAirSupport then
			if not TRAIN_CHASE_SPAWN_CONTROL then
				TRAIN_CHASE_SPAWN_CONTROL = true
				self.turret[i] = CreateACrab("Turret Control","Air Support.rte");
			else
				self.turret[i] = CreateACrab(self.turrets[math.random(#self.turrets)],"Air Support.rte");
			end
		end
		
		-- Vanilla turret fallback
		if self.turret[i] == nil then
			self.turret[i] = CreateACrab("Gatling Turret AA", "TrainChase.rte");
		end
		
		if i == 1 then
			self.turret[i].Pos = Vector(self.Pos.X + 40,self.Pos.Y - 44) + self.Offset;
		end

		if i == 2 then
			self.turret[i].Pos = Vector(self.Pos.X - 40,self.Pos.Y - 44) + self.Offset;
		end
		
		self.turret[i].Vel = self.Vel;
		self.turret[i].Team = self.Team;
		MovableMan:AddActor(self.turret[i]);
	end
end

function Update(self)
    --If the train hasn't crashed...
    if self.crashed == false then
	--The following lines level out the train (until it has crashed).
	--This makes it hover.
	self.Vel.Y = self.Vel.Y - ((self.Pos.Y - self.hoverY)) / 35;

	--This makes sure it's hovering around the right spot.
	self.Pos.Y = ((self.Pos.Y * 9) + self.hoverY) / 10;

	--This keeps the train at a constant speed.
	if self.Vel.X ~= 50 then
	    self.Vel.X = (self.Vel.X * 9 + self.curspeed) / 10;
	end

	--This keeps the train rotated in the right direction.
	if self.RotAngle ~= 0 then
	    self.AngularVel = (self.AngularVel * 74 - math.deg(self.RotAngle)) / 75;
	end

	--Crash the train if it's rotated too far.
	if self.RotAngle <= -self.maxrot or self.RotAngle >= self.maxrot then
	    self.crashed = true;
	    self.crashtimer:Reset();
	end

	--Slow down and stop if at a stop area.
	if self.stoparea ~= nil then
	    --Find the area to stop in.
	    stopbox = self.stoparea:GetBoxInside(self.Pos);
	    if stopbox ~= nil then
		--If we're not at a complete stop yet, slow down with a nice pattern.
		if self.stopped == false then
		    if self.Pos.X < stopbox.Center.X then
			--Math to make a nice slowing gradient.
			self.curspeed = ((stopbox.Center.X - self.Pos.X) / (stopbox.Width / 2)) * self.maxspeed;
			--Reset the stop timer; we haven't stopped yet.
			self.stoptimer:Reset();
			--If the train is in approximately the right position, flag that it is stopped.
			if (self.Pos.X > stopbox.Center.X - 5) and (self.Pos.X < stopbox.Center.X + 5) then
			    self.curspeed = 0;
			    self.stopped = true;
			end
		    end
		--Otherwise, after 7.5 seconds, go.
		elseif self.stoptimer:IsPastSimMS(7500) then
		    --If the train is not past the center, give it a nudge so it can start going.
		    if self.Pos.X - stopbox.Center.X <= 0 then
			self.Pos.X = self.Pos.X + 2;
		    end
		    --Increase the speed in an increasing gradient as it leaves.
		    self.curspeed = ((self.Pos.X - stopbox.Center.X) / (stopbox.Width / 2)) * self.maxspeed;
		    --If the train is behind the center, it will go backward.  To avoid this, we reverse the speed if it is negative.
		    if self.curspeed < 0 then
			self.curspeed = self.curspeed * -1 + 2;
		    end
		    --Once we're close enough to the maximum speed, flag that the train is no longer stopped and cap off the speed to what it should be.
		    if self.curspeed >= self.maxspeed - 3 then
			self.curspeed = self.maxspeed;
			self.stopped = false;
		    end
		end
	    end
	end
	
	--If the train is dead, crash it.
	if self.Health <= 100 then
	    self.crashed = true;
	    self.crashtimer:Reset();
	end

	--Every 5 seconds, check if the train has any sway.  If not, apply some.
	if self.swaytimer:IsPastSimMS(1000) then
	    self.swaytimer:Reset();
	    self.AngularVel = self.sway;
	end
    elseif self.crashtimer:IsPastSimMS(5000) == true then
	--Explode five seconds after crashing.
	self:GibThis();
    end
	
	
	for i = 1,2 do
		if MovableMan:IsActor(self.turret[i]) and MovableMan:IsActor(self) then

			if i == 1 then
				local x = math.sin(self.RotAngle + 1.57 * 1.53) * 60;
				local y = math.cos(self.RotAngle + 1.57 * 1.53) * 60;
				
				self.turret[i].Pos = Vector(self.Pos.X + x, self.Pos.Y + y) + self.Offset;
			end

			if i == 2 then
				local x = math.sin(self.RotAngle - 1.57 * 1.53) * 60;
				local y = math.cos(self.RotAngle - 1.57 * 1.53) * 60;
				
				self.turret[i].Pos = Vector(self.Pos.X + x, self.Pos.Y + y) + self.Offset;
			end
			
			self.turret[i].Vel = self.Vel;
			self.turret[i].RotAngle = self.RotAngle;
		end
		
		if not MovableMan:IsActor(self.turret[i]) and MovableMan:IsActor(self) then
			self.turret[i] = nil;
			if math.random() < 0.005 then
				if self.UseUniTec then
					self.turret[i] = CreateACrab(self.turrets[math.random(#self.turrets)],"UniTec.rte");
				end
				
				if self.UseAirSupport then
					self.turret[i] = CreateACrab(self.turrets[math.random(#self.turrets)],"Air Support.rte");
				end
				
				-- Vanilla turret fallback
				if self.turret[i] == nil then
					self.turret[i] = CreateACrab("Gatling Turret AA", "TrainChase.rte");
				end
				
				if self.turret[i] ~= nil then
					if i == 1 then
						self.turret[i].Pos = Vector(self.Pos.X + 40,self.Pos.Y - 44) + self.Offset;
					end

					if i == 2 then
						self.turret[i].Pos = Vector(self.Pos.X - 40,self.Pos.Y - 44) + self.Offset;
					end
					
					self.turret[i].Vel = self.Vel;
					self.turret[i].Team = self.Team;
					MovableMan:AddActor(self.turret[i]);
				end
			end
		end
	end
end

function Destroy(self)
	for i = 1,2 do
		if MovableMan:IsActor(self.turret[i]) then
			self.turret[i].Health = 0
		end
	end
end