-----------------------------------------------------------------------------------------
-- Start Activity
-----------------------------------------------------------------------------------------
function TrainChase:StartActivity()
	print("START! -- TrainChase:StartActivity()!");
	
	self.braindead = {};
	self.AssignedBrains = {};

	self.MOIDLimit = 200;

	self.PlayerGold = 150000;

    self.BrainArea = SceneMan.Scene:GetArea("Brain");
	
   	self.CPUTeam = Activity.TEAM_2;
	self.PlayerTeam = Activity.TEAM_1;
	self.RogueTeam = -1;

	self.Train = nil
	
	self.Carts = {}
	self.CartsNum = 3
	
	self.AlertTriggered = false;
	
	self.GenericTimer = Timer();
	self.GenericTimer:Reset();
	
	self.EnumEven = true;
	
	--Spawn Our Heroes
	self.Brain = CreateActor("Brain Case" , "Base.rte");
	self.Brain.Team = self.PlayerTeam;
	self.Brain.AIMode = Actor.AIMODE_SENTRY;
	self.Brain.HitsMOs = false;
	self.Brain.GetsHitByMOs = false;
	self.Brain.Scale = 0
	self.Brain.Pos = self.BrainArea:GetCenterPoint();
	for player = 0, self.PlayerCount - 1 do
		self:SetPlayerBrain(self.Brain, player);
	end
	MovableMan:AddActor(self.Brain)
	
	self:SetTeamFunds(self.PlayerGold , self.PlayerTeam);
	
	TRAIN_CHASE_SPAWN_CONTROL = false;
	
	self:AdvanceToBoss();
end
-----------------------------------------------------------------------------------------
-- Pause Activity
-----------------------------------------------------------------------------------------
-- This function is called when mission is paused
function TrainChase:PauseActivity(pause)
    print("PAUSE! -- TrainChase:PauseActivity()!");
end
-----------------------------------------------------------------------------------------
-- End Activity
-----------------------------------------------------------------------------------------
-- This function is called after mission has ended
function TrainChase:EndActivity()
    print("END! -- TrainChase:EndActivity()!");
end
-----------------------------------------------------------------------------------------
-- Update Activity
-----------------------------------------------------------------------------------------
function TrainChase:UpdateActivity()
   	self:ClearObjectivePoints();
	self:DoBrainSelection()
	self:PhaseBoss();
	self:YSortObjectivePoints();
end
-----------------------------------------------------------------------------------------
--
-----------------------------------------------------------------------------------------
function TrainChase:AdvanceToBoss()
	print ("AdvanceToBoss")

	self.GenericTimer:Reset();

	self.Cart1Pos = SceneMan.Scene:GetArea("DropZone"):GetCenterPoint();

	self.Train = CreateACRocket("Train Locomotive" , "TrainChase.rte");
	self.Train.Pos = self.Cart1Pos;
	self.Train.Team = self.CPUTeam;
	self.Train.AIMode = Actor.AIMODE_STAY;
	MovableMan:AddActor(self.Train);

	for i = 1, self.CartsNum do
		self.Carts[i] = CreateACRocket("Train Cart" , "TrainChase.rte");
		self.Carts[i].Pos = self.Cart1Pos + Vector(-200 * i, 0)
		self.Carts[i].Team = self.CPUTeam;
		self.Carts[i].AIMode = Actor.AIMODE_STAY;
		MovableMan:AddActor(self.Carts[i]);
	end
end
-----------------------------------------------------------------------------------------
--
-----------------------------------------------------------------------------------------
function TrainChase:PhaseBoss()
	-- Process locomotive
	if MovableMan:IsActor(self.Train) then
		self:AddObjectivePoint("Kill", self.Train.AboveHUDPos, self.PlayerTeam, GameActivity.ARROWDOWN);
	else
		print ("Train dead")
		self.Train = nil;

		if MovableMan:IsActor(self.Carts[i]) then
			self.Carts[i]:GibThis();
		end

		if MovableMan:IsActor(self.Train3) then
			self.Train3:GibThis();
		end
		
		if self.ActivityState ~= Activity.OVER then
			self.WinnerTeam = self.PlayerTeam;
			ActivityMan:EndActivity();
			for player = 0, self.PlayerCount - 1 do
				self.braindead[player] = true;
				FrameMan:ClearScreenText(player);
			end
		end
	end

	for i = 1, self.CartsNum do
		-- Process carts
		if not MovableMan:IsActor(self.Carts[i]) then
			self.Carts[i] = nil;
		else
			-- Correct cart speed
			-- First find nearest cart
			local leader = nil;
			
			for j = i - 1 , 1 , -1 do
				if MovableMan:IsActor(self.Carts[j]) then
					leader = self.Carts[j]
					break;
				end
			end
			
			if leader == nil then
				if MovableMan:IsActor(self.Train) then
					leader = self.Train
				end
			end
			
			--if i == 2 then
			--	print (leader)
			--end
			
			if leader ~= nil then
				local trainvelcorrection = nil;
				local dist = SceneMan:ShortestDistance(leader.Pos, self.Carts[i].Pos, true)
			
				if dist.Magnitude < 199 then
					trainvelcorrection = Vector(-((200 - math.abs(dist.X)) / 50), 0)
				end

				if dist.Magnitude > 201 then
					trainvelcorrection = Vector((math.abs(dist.X) - 200) / 50, 0)
				end
				
				if trainvelcorrection ~= nil then
					self.Carts[i].Vel = Vector(leader.Vel.X, self.Carts[i].Vel.Y) + trainvelcorrection;
				else
					self.Carts[i].Vel = Vector(leader.Vel.X, self.Carts[i].Vel.Y);
				end
			end
		end
	end
end
-----------------------------------------------------------------------------------------
-- Brain selection and gameover conditions check
-----------------------------------------------------------------------------------------
function TrainChase:DoBrainSelection()
	if self.ActivityState ~= Activity.OVER then
		for player = 0, self.PlayerCount - 1 do
			local team = self:GetTeamOfPlayer(player);

			local brain = self:GetPlayerBrain(player);
			if not brain or not MovableMan:IsActor(brain) then
				self:SetPlayerBrain(nil, player);
				local newBrain = MovableMan:GetUnassignedBrain(team);
				if newBrain and self.braindead[player] == false then
					self:SetPlayerBrain(newBrain, player);
					self.AssignedBrains[player] = newBrain;
					self:SwitchToActor(newBrain, player, team);
				end
			else
				self:SetObservationTarget(brain.Pos, player)
			end
		end
	end
end
-----------------------------------------------------------------------------------------
-- That's all folks!
-----------------------------------------------------------------------------------------



































